@@ -263,72 +263,89 @@ desktop_entry_load_directory (DesktopEntry  *entry,
   return TRUE;
 }
 
-static gboolean
+static DesktopEntryResultCode
 desktop_entry_load (DesktopEntry *entry)
 {
+  DesktopEntryResultCode rescode = DESKTOP_ENTRY_LOAD_FAIL_OTHER;
+
   if (strstr (entry->path, "/menu-xdg/"))
-    return FALSE;
+    return rescode;
+
   if (entry->type == DESKTOP_ENTRY_DESKTOP)
     {
       GKeyFile *key_file = NULL;
       DesktopEntryDesktop *entry_desktop = (DesktopEntryDesktop*)entry;
-      const char *categories_str;
-
-      entry_desktop->appinfo = g_desktop_app_info_new_from_filename (entry->path);
-      if (!entry_desktop->appinfo ||
-          !g_app_info_get_name (G_APP_INFO (entry_desktop->appinfo)) ||
-          !g_app_info_get_executable (G_APP_INFO (entry_desktop->appinfo)))
-        {
-          menu_verbose ("Failed to load \"%s\"\n", entry->path);
-          return FALSE;
-        }
-
-      categories_str = g_desktop_app_info_get_categories (entry_desktop->appinfo);
-      if (categories_str)
-        {
-          char **categories;
-          int i;
-
-          categories = g_strsplit (categories_str, ";", -1);
-          entry_desktop->categories = g_new0 (GQuark, g_strv_length (categories) + 1);
-
-          for (i = 0; categories[i]; i++)
-            entry_desktop->categories[i] = g_quark_from_string (categories[i]);
-
-          g_strfreev (categories);
-        }
 
       key_file = g_key_file_new ();
 
-      if (!g_key_file_load_from_file (key_file, entry->path, 0, NULL))
-        entry_desktop->showin = TRUE;
+      if (g_key_file_load_from_file (key_file, entry->path, 0, NULL))
+        {
+          entry_desktop->appinfo = g_desktop_app_info_new_from_keyfile (key_file);
+
+          if (!entry_desktop->appinfo ||
+              !g_app_info_get_name (G_APP_INFO (entry_desktop->appinfo)) ||
+              !g_app_info_get_executable (G_APP_INFO (entry_desktop->appinfo)))
+            {
+              menu_verbose ("Failed to load appinfo for \"%s\"\n", entry->path);
+              rescode = DESKTOP_ENTRY_LOAD_FAIL_APPINFO;
+            }
+          else
+            {
+              const char *categories_str;
+              categories_str = g_desktop_app_info_get_categories (entry_desktop->appinfo);
+
+              if (categories_str)
+                {
+                  char **categories;
+                  int i;
+
+                  categories = g_strsplit (categories_str, ";", -1);
+                  entry_desktop->categories = g_new0 (GQuark, g_strv_length (categories) + 1);
+
+                  for (i = 0; categories[i]; i++)
+                    entry_desktop->categories[i] = g_quark_from_string (categories[i]);
+
+                  g_strfreev (categories);
+                }
+
+              entry_desktop->showin = key_file_get_show_in (key_file);
+
+              rescode = DESKTOP_ENTRY_LOAD_SUCCESS;
+            }
+        }
       else
-        entry_desktop->showin = key_file_get_show_in (key_file);
-
+        {
+          menu_verbose ("Failed to read contents of \"%s\"\n", entry->path);
+          rescode = DESKTOP_ENTRY_LOAD_FAIL_OTHER;
+        }
       g_key_file_free (key_file);
-
-      return TRUE;
     }
   else if (entry->type == DESKTOP_ENTRY_DIRECTORY)
     {
       GKeyFile *key_file = NULL;
       GError   *error = NULL;
-      gboolean  retval = FALSE;
+      rescode = DESKTOP_ENTRY_LOAD_SUCCESS;
 
       key_file = g_key_file_new ();
 
       if (!g_key_file_load_from_file (key_file, entry->path, 0, &error))
-        goto out;
+        {
+          rescode = DESKTOP_ENTRY_LOAD_FAIL_OTHER;
+          goto out;
+        }
 
       if (!desktop_entry_load_directory (entry, key_file, &error))
-        goto out;
+        {
+          rescode = DESKTOP_ENTRY_LOAD_FAIL_OTHER;
+          goto out;
+        }
 
-      retval = TRUE;
+      rescode = DESKTOP_ENTRY_LOAD_SUCCESS;
 
     out:
       g_key_file_free (key_file);
 
-      if (!retval)
+      if (rescode == DESKTOP_ENTRY_LOAD_FAIL_OTHER)
         {
           if (error)
             {
@@ -338,20 +355,27 @@ desktop_entry_load (DesktopEntry *entry)
           else
             menu_verbose ("Failed to load \"%s\"\n", entry->path);
         }
-
-      return retval;
     }
   else
     g_assert_not_reached ();
 
-  return FALSE;
+  return rescode;
+}
+
+static gboolean
+code_failed (DesktopEntryResultCode code)
+{
+    return code == DESKTOP_ENTRY_LOAD_FAIL_OTHER ||
+           code == DESKTOP_ENTRY_LOAD_FAIL_APPINFO;
 }
 
 DesktopEntry *
-desktop_entry_new (const char *path)
+desktop_entry_new (const char             *path,
+                   DesktopEntryResultCode *res_code)
 {
   DesktopEntryType  type;
   DesktopEntry     *retval;
+  DesktopEntryResultCode code;
 
   menu_verbose ("Loading desktop entry \"%s\"\n", path);
 
@@ -369,6 +393,7 @@ desktop_entry_new (const char *path)
     {
       menu_verbose ("Unknown desktop entry suffix in \"%s\"\n",
                     path);
+      *res_code = DESKTOP_ENTRY_LOAD_FAIL_OTHER;
       return NULL;
     }
 
@@ -377,7 +402,10 @@ desktop_entry_new (const char *path)
   retval->path     = g_strdup (path);
   retval->basename = unix_basename_from_path (retval->path);
 
-  if (!desktop_entry_load (retval))
+  code = desktop_entry_load (retval);
+  *res_code = code;
+
+  if (code_failed (code))
     {
       desktop_entry_unref (retval);
       return NULL;
@@ -419,7 +447,7 @@ desktop_entry_reload (DesktopEntry *entry)
   else
     g_assert_not_reached ();
 
-  if (!desktop_entry_load (entry))
+  if (code_failed (desktop_entry_load (entry)))
     {
       desktop_entry_unref (entry);
       return NULL;
